@@ -9,11 +9,11 @@ File::Next - File-finding iterator
 
 =head1 VERSION
 
-Version 1.08
+Version 1.10
 
 =cut
 
-our $VERSION = '1.08';
+our $VERSION = '1.10';
 
 =head1 SYNOPSIS
 
@@ -88,15 +88,15 @@ in I<$filename>.  If I<$filename> is C<->, then the files are read
 from STDIN.
 
 The files are assumed to be in the file one filename per line.  If
-I<$null_separated> is passed, then the files are assumed to be
+I<$nul_separated> is passed, then the files are assumed to be
 NUL-separated, as by C<find -print0>.
 
-If there are blank lines or null filenames in the input stream,
+If there are blank lines or empty filenames in the input stream,
 they are ignored.
 
 Each filename is checked to see that it is a regular file or a named
-pipe.  If the file does not exists or is a directory, then it is
-skipped.
+pipe.  If the file does not exists or is a directory, then a warning
+is thrown to I<warning_handler>, and the file is skipped.
 
 The following options have no effect in C<from_files>: I<descend_filter>,
 I<sort_files>, I<follow_symlinks>.
@@ -193,7 +193,13 @@ By default, the I<descend_filter> is C<sub {1}>, or "always descend".
 
 If I<error_handler> is set, then any errors will be sent through
 it.  By default, this value is C<CORE::die>.  This function must
-not return.
+NOT return.
+
+=head2 warning_handler => \&warning_handler
+
+If I<warning_handler> is set, then any errors will be sent through
+it.  By default, this value is C<CORE::warn>.  Unlike the
+I<error_handler>, this function must return.
 
 =head2 sort_files => [ 0 | 1 | \&sort_sub]
 
@@ -241,6 +247,7 @@ BEGIN {
         file_filter     => undef,
         descend_filter  => undef,
         error_handler   => sub { CORE::die @_ },
+        warning_handler => sub { CORE::warn @_ },
         sort_files      => undef,
         follow_symlinks => 1,
         nul_separated   => 0,
@@ -250,7 +257,7 @@ BEGIN {
 
 
 sub files {
-    die _bad_invocation() if $_[0] eq __PACKAGE__;
+    die _bad_invocation() if @_ && defined($_[0]) && ($_[0] eq __PACKAGE__);
 
     my ($parms,@queue) = _setup( \%files_defaults, @_ );
     my $filter = $parms->{file_filter};
@@ -278,7 +285,7 @@ sub files {
 
 
 sub dirs {
-    die _bad_invocation() if $_[0] eq __PACKAGE__;
+    die _bad_invocation() if @_ && defined($_[0]) && ($_[0] eq __PACKAGE__);
 
     my ($parms,@queue) = _setup( \%files_defaults, @_ );
 
@@ -296,7 +303,7 @@ sub dirs {
 }
 
 sub everything {
-    die _bad_invocation() if $_[0] eq __PACKAGE__;
+    die _bad_invocation() if @_ && defined($_[0]) && ($_[0] eq __PACKAGE__);
 
     my ($parms,@queue) = _setup( \%files_defaults, @_ );
     my $filter = $parms->{file_filter};
@@ -321,15 +328,17 @@ sub everything {
 }
 
 sub from_file {
-    die _bad_invocation() if $_[0] eq __PACKAGE__;
+    die _bad_invocation() if @_ && defined($_[0]) && ($_[0] eq __PACKAGE__);
 
     my ($parms,@queue) = _setup( \%files_defaults, @_ );
-    my $err = $parms->{error_handler};
+    my $err  = $parms->{error_handler};
+    my $warn = $parms->{error_handler};
 
     my $filename = $queue[1];
 
-    if ( !$filename ) {
+    if ( !defined($filename) ) {
         $err->( 'Must pass a filename to from_file()' );
+        return undef;
     }
 
     my $fh;
@@ -337,9 +346,9 @@ sub from_file {
         $fh = \*STDIN;
     }
     else {
-        open( $fh, '<', $filename );
-        if ( !$fh ) {
+        if ( !open( $fh, '<', $filename ) ) {
             $err->( "Unable to open $filename: $!" );
+            return undef;
         }
     }
     my $filter = $parms->{file_filter};
@@ -349,7 +358,10 @@ sub from_file {
         while ( my $fullpath = <$fh> ) {
             chomp $fullpath;
             next unless $fullpath =~ /./;
-            next unless -f $fullpath || -p _;
+            if ( not ( -f $fullpath || -p _ ) ) {
+                $warn->( "$fullpath: No such file" );
+                next;
+            }
 
             my ($volume,$dirname,$file) = File::Spec->splitpath( $fullpath );
             if ( $filter ) {
@@ -581,6 +593,12 @@ L<http://github.com/petdance/file-next/tree/master>
 All file-finding in this module is adapted from Mark Jason Dominus'
 marvelous I<Higher Order Perl>, page 126.
 
+Thanks also for bug fixes and typo finding to
+Bruce Woodward,
+Christopher J. Madsen,
+Bernhard Fisseni
+and Rob Hoelz.
+
 =head1 COPYRIGHT & LICENSE
 
 Copyright 2005-2012 Andy Lester.
